@@ -207,7 +207,7 @@ class BCBase(metaclass=ABCMeta):
             rank (int):
                 The tensorial rank of the value associated with the boundary
                 condition.
-            value (float or str or array):
+            value (float or str or :class:`~numpy.ndarray`):
                 a value stored with the boundary condition. The interpretation
                 of this value depends on the type of boundary condition. If
                 value is a single value (or tensor in case of tensorial boundary
@@ -387,16 +387,13 @@ class BCBase(metaclass=ABCMeta):
     
     def __repr__(self):
         if self.value_is_linked:
-            return (f"{self.__class__.__name__}("
-                    f"axis={self.axis}, upper={self.upper}, "
-                    f"value=<linked: {self.value.ctypes.data}>)")
+            value_str = f", value=<linked: {self.value.ctypes.data}>"
         elif np.array_equal(self.value, 0):
-            return (f"{self.__class__.__name__}("
-                    f"axis={self.axis}, upper={self.upper}, rank={self.rank})")
+            value_str = ""
         else:
-            return (f"{self.__class__.__name__}("
-                    f"axis={self.axis}, upper={self.upper}, "
-                    f"value={self.value!r})")
+            value_str = f", value={self.value!r}"
+        return (f"{self.__class__.__name__}(axis={self.axis}, "
+                f"upper={self.upper}, rank={self.rank}{value_str})")
     
     
     def __str__(self):
@@ -432,18 +429,16 @@ class BCBase(metaclass=ABCMeta):
     def _cache_hash(self) -> int:
         """ returns a value to determine when a cache needs to be updated """ 
         if self.value_is_linked:
-            return hash((self.__class__.__name__,
-                         self.grid._cache_hash(),
-                         self.axis))
+            value = self.value.ctypes.data  # type: ignore
+        elif isinstance(self.value, np.ndarray):
+            value = self.value.tobytes()
         else:
-            if isinstance(self.value, np.ndarray):
-                value = self.value.tobytes()
-            else:
-                value = self.value
-            return hash((self.__class__.__name__,
-                         self.grid._cache_hash(),
-                         self.axis,
-                         value))
+            value = self.value
+
+        return hash((self.__class__.__name__,
+                     self.grid._cache_hash(),
+                     self.axis,
+                     value))
 
 
     def copy(self, upper: Optional[bool] = None,
@@ -681,7 +676,7 @@ class BCBase1stOrder(BCBase):
 
     @abstractmethod
     def get_virtual_point_data(self, compiled: bool = False) \
-        -> Tuple: pass
+        -> Tuple[Any, Any, int]: pass
 
 
     def get_data(self, idx: Tuple[int, ...]) -> Tuple[float, Dict[int, float]]:
@@ -809,6 +804,8 @@ class BCBase1stOrder(BCBase):
             
             # calculate necessary constants
             const, factor, index = self.get_virtual_point_data(compiled=True)
+            zeros = np.zeros(self._shape_tensor)
+            ones = np.ones(self._shape_tensor)
 
             @register_jitable(inline='always')
             def adjacent_point(arr_1d, i_point, bc_idx) -> float:
@@ -820,7 +817,7 @@ class BCBase1stOrder(BCBase):
                 if i_point == i_bndry:
                     c, f, i = const(), factor(), index
                 else:
-                    c, f, i = 0., 1., i_point + i_dx  # INTENTIONAL; see above
+                    c, f, i = zeros, ones, i_point + i_dx  # INTENTIONAL
                 
                 # calculate the values
                 return c + f * arr_1d[..., i]  # type: ignore
@@ -830,9 +827,8 @@ class BCBase1stOrder(BCBase):
 
             # calculate necessary constants
             const, factor, index = self.get_virtual_point_data(compiled=True)
-            
-            shape = self._shape_tensor + self._shape_boundary
-            zeros, ones = np.zeros(shape), np.ones(shape)
+            zeros = np.zeros(self._shape_tensor + self._shape_boundary)
+            ones = np.ones(self._shape_tensor + self._shape_boundary)
             
             @register_jitable(inline='always')
             def adjacent_point(arr_1d, i_point, bc_idx) -> float:
@@ -861,7 +857,8 @@ class DirichletBC(BCBase1stOrder):
     names = ['value', 'dirichlet']  # identifiers for this boundary condition
 
     
-    def get_virtual_point_data(self, compiled: bool = False) -> Tuple:
+    def get_virtual_point_data(self, compiled: bool = False) \
+            -> Tuple[Any, Any, int]:
         """ return data suitable for calculating virtual points
         
         Args:
@@ -875,10 +872,10 @@ class DirichletBC(BCBase1stOrder):
             virtual point
         """        
         const = 2 * self.value
-        factor = -1 if np.isscalar(const) else -np.ones_like(const)
         index = self.grid.shape[self.axis] - 1 if self.upper else 0
             
         if not compiled:
+            factor = -1 if np.isscalar(const) else -np.ones_like(const)
             return (const, factor, index)
         else:
             # return boundary data such that dynamically calculated values can
@@ -886,6 +883,9 @@ class DirichletBC(BCBase1stOrder):
             # arrays are copied into closures, making them compile-time
             # constants
             
+            const = np.array(const, np.double)
+            factor = np.full_like(const, -1)
+            
             if self.value_is_linked:
                 value = self._make_value_getter()
                 
@@ -919,7 +919,8 @@ class NeumannBC(BCBase1stOrder):
     names = ['derivative', 'neumann']  # identifiers for this boundary condition        
 
     
-    def get_virtual_point_data(self, compiled: bool = False) -> Tuple:
+    def get_virtual_point_data(self, compiled: bool = False) \
+            -> Tuple[Any, Any, int]:
         """ return data suitable for calculating virtual points
         
         Args:
@@ -935,10 +936,10 @@ class NeumannBC(BCBase1stOrder):
         dx = self.grid.discretization[self.axis]
         
         const = dx * self.value
-        factor = 1 if np.isscalar(const) else np.ones_like(const)
         index = self.grid.shape[self.axis] - 1 if self.upper else 0
             
         if not compiled:
+            factor = 1 if np.isscalar(const) else np.ones_like(const)
             return (const, factor, index)
         else:
             # return boundary data such that dynamically calculated values can
@@ -946,6 +947,9 @@ class NeumannBC(BCBase1stOrder):
             # arrays are copied into closures, making them compile-time
             # constants
             
+            const = np.array(const, np.double)
+            factor = np.ones_like(const)
+            
             if self.value_is_linked:
                 value = self._make_value_getter()
                 
@@ -1067,7 +1071,8 @@ class MixedBC(BCBase1stOrder):
         return obj
         
         
-    def get_virtual_point_data(self, compiled: bool = False) -> Tuple:
+    def get_virtual_point_data(self, compiled: bool = False) \
+            -> Tuple[Any, Any, int]:
         """ return data suitable for calculating virtual points
         
         Args:
@@ -1109,7 +1114,7 @@ class MixedBC(BCBase1stOrder):
         # constants
 
         if self.value_is_linked:
-            const_val = float(self.const)  # only support scalar values
+            const_val = np.array(self.const, np.double)
             value_func = self._make_value_getter()
             
             @register_jitable(inline='always')
@@ -1137,6 +1142,9 @@ class MixedBC(BCBase1stOrder):
                 return factor
             
         else:
+            const = np.array(const, np.double)
+            factor = np.array(factor, np.double)
+            
             @register_jitable(inline='always')
             def const_func():
                 return const          
