@@ -4,7 +4,6 @@
 
 import os
 from unittest import mock
-import tempfile
 import pickle
 
 import numpy as np
@@ -146,23 +145,24 @@ def test_callback_tracker():
    
     
 
-def test_data_tracker():
+def test_data_tracker(tmp_path):
     """ test the DataTracker """
     field = ScalarField(UnitGrid([4, 4]))
     eq = DiffusionPDE()
     
-    fp = tempfile.NamedTemporaryFile(suffix='.pickle')
-    data1 = trackers.DataTracker(lambda f: f.average, filename=fp.name)
+    path = tmp_path / "test_data_tracker.pickle"
+    data1 = trackers.DataTracker(lambda f: f.average, filename=path)
     data2 = trackers.DataTracker(lambda f: {'avg': f.average,
                                             'int': f.integral})
     eq.solve(field, 10, tracker=[data1, data2])
     
-    time, data = pickle.load(fp)
+    with path.open('br') as fp:
+        time, data = pickle.load(fp)
     np.testing.assert_allclose(time, np.arange(11))
     assert isinstance(data, list)
     assert len(data) == 11
     
-    assert os.stat(fp.name).st_size > 0
+    assert path.stat().st_size > 0
      
     
     
