@@ -2,9 +2,8 @@ import pytest
 import numpy as np
 
 
-
 @pytest.fixture(scope="session", autouse=True)
-def enable_raise_errors():
-    """ helper function enabling errors for all tests """
-    # prepare something ahead of all tests
-    np.seterr(all='raise', under='ignore')
\ No newline at end of file
+def adjust_messages():
+    """ helper function adjusting message reporting for all tests """
+    # raise all underflow errors
+    np.seterr(all='raise', under='ignore')
